@@ -571,7 +571,7 @@ Erase it? Y/N: """))
         return 0
 
 
-    def do_set(self, args, pwre = re.compile(r'{(\w+)}(.+)')):
+    def do_set(self, args):
         ""'''Usage: set items property=value property=value ...
         Set the given properties of one or more items(s).
 
@@ -732,7 +732,7 @@ Erase it? Y/N: """))
                 value = cl.get(nodeid, key)
                 print _('%(key)s: %(value)r')%locals()
 
-    def do_create(self, args, pwre = re.compile(r'{(\w+)}(.+)')):
+    def do_create(self, args):
         ""'''Usage: create classname property=value ...
         Create a new entry of a given class.
 
@@ -1068,7 +1068,7 @@ Erase it? Y/N: """))
         for classname in classes:
             cl = self.get_class(classname)
 
-            f = open(os.path.join(dir, classname+'.csv'), 'w')
+            f = open(os.path.join(dir, classname+'.csv'), 'wb')
             writer = csv.writer(f, colon_separated)
 
             properties = cl.getprops()
@@ -1087,7 +1087,7 @@ Erase it? Y/N: """))
             f.close()
 
             # export the journals
-            jf = open(os.path.join(dir, classname+'-journals.csv'), 'w')
+            jf = open(os.path.join(dir, classname+'-journals.csv'), 'wb')
             journals = csv.writer(jf, colon_separated)
             map(journals.writerow, cl.export_journals())
             jf.close()
@@ -1151,7 +1151,7 @@ Erase it? Y/N: """))
             f.close()
 
             # import the journals
-            f = open(os.path.join(args[0], classname + '-journals.csv'), 'rU')
+            f = open(os.path.join(args[0], classname + '-journals.csv'), 'r')
             reader = csv.reader(f, colon_separated)
             cl.import_journals(reader)
             f.close()
@@ -1251,11 +1251,16 @@ Erase it? Y/N: """))
         for rolename, role in roles:
             print _('Role "%(name)s":')%role.__dict__
             for permission in role.permissions:
+                d = permission.__dict__
                 if permission.klass:
-                    print _(' %(description)s (%(name)s for "%(klass)s" '
-                        'only)')%permission.__dict__
+                    if permission.properties:
+                        print _(' %(description)s (%(name)s for "%(klass)s"'
+                          ': %(properties)s only)')%d
+                    else:
+                        print _(' %(description)s (%(name)s for "%(klass)s" '
+                            'only)')%d
                 else:
-                    print _(' %(description)s (%(name)s)')%permission.__dict__
+                    print _(' %(description)s (%(name)s)')%d
         return 0
 
     def run_command(self, args):
