@@ -1431,16 +1431,15 @@ class Class(hyperdb.Class):
         '''Get the ids of items in this class which link to the given items.
 
         'propspec' consists of keyword args propname=itemid or
-                   propname={itemid:1, }
+                   propname={<itemid 1>:1, <itemid 2>: 1, ...}
         'propname' must be the name of a property in this class, or a
                    KeyError is raised.  That property must be a Link or
                    Multilink property, or a TypeError is raised.
 
-        Any item in this class whose 'propname' property links to any of the
-        itemids will be returned. Used by the full text indexing, which knows
-        that "foo" occurs in msg1, msg3 and file7, so we have hits on these
-        issues:
+        Any item in this class whose 'propname' property links to any of
+        the itemids will be returned. Examples::
 
+            db.issue.find(messages='1')
             db.issue.find(messages={'1':1,'3':1}, files={'7':1})
         '''
         propspec = propspec.items()
@@ -1589,7 +1588,6 @@ class Class(hyperdb.Class):
         INTERVAL = 'spec:interval'
         OTHER = 'spec:other'
 
-        timezone = self.db.getUserTimezone()
         for k, v in filterspec.items():
             propclass = props[k]
             if isinstance(propclass, hyperdb.Link):
@@ -1620,7 +1618,7 @@ class Class(hyperdb.Class):
                     l.append((STRING, k, re.compile(v, re.I)))
             elif isinstance(propclass, hyperdb.Date):
                 try:
-                    date_rng = date.Range(v, date.Date, offset=timezone)
+                    date_rng = propclass.range_from_raw (v, self.db)
                     l.append((DATE, k, date_rng))
                 except ValueError:
                     # If range creation fails - ignore that search parameter
@@ -2029,7 +2027,9 @@ class Class(hyperdb.Class):
                             if not isinstance(value, type('')):
                                 value = value.get_tuple()
                         elif isinstance(prop, hyperdb.Interval):
-                            value = value.get_tuple()
+                            # hack too - some intervals are stored as strings
+                            if not isinstance(value, type('')):
+                                value = value.get_tuple()
                         elif isinstance(prop, hyperdb.Password):
                             value = str(value)
                         export_data[propname] = value
