@@ -1082,20 +1082,18 @@ class Class(hyperdb.Class):
             self.db.dirty = 1
 
     def find(self, **propspec):
-        '''Get the ids of nodes in this class which link to the given nodes.
+        '''Get the ids of items in this class which link to the given items.
 
-        'propspec'
-             consists of keyword args propname={nodeid:1,}
-        'propname'
-             must be the name of a property in this class, or a
-             KeyError is raised.  That property must be a Link or
-             Multilink property, or a TypeError is raised.
+        'propspec' consists of keyword args propname=itemid or
+                   propname={<itemid 1>:1, <itemid 2>: 1, ...}
+        'propname' must be the name of a property in this class, or a
+                   KeyError is raised.  That property must be a Link or
+                   Multilink property, or a TypeError is raised.
 
-        Any node in this class whose propname property links to any of the
-        nodeids will be returned. Used by the full text indexing, which knows
-        that "foo" occurs in msg1, msg3 and file7; so we have hits on these
-        issues::
+        Any item in this class whose 'propname' property links to any of
+        the itemids will be returned. Examples::
 
+            db.issue.find(messages='1')
             db.issue.find(messages={'1':1,'3':1}, files={'7':1})
         '''
         propspec = propspec.items()
@@ -1211,8 +1209,6 @@ class Class(hyperdb.Class):
         if __debug__:
             start_t = time.time()
 
-        timezone = self.db.getUserTimezone()
-
         where = {'_isdel':0}
         wherehigh = {}
         mlcriteria = {}
@@ -1279,7 +1275,7 @@ class Class(hyperdb.Class):
             elif isinstance(prop, hyperdb.Date):
                 try:
                     # Try to filter on range of dates
-                    date_rng = Range(value, date.Date, offset=timezone)
+                    date_rng = prop.range_from_raw (value, self.db)
                     if date_rng.from_value:
                         t = date_rng.from_value.get_tuple()
                         where[propname] = int(calendar.timegm(t))
@@ -1706,7 +1702,12 @@ class Class(hyperdb.Class):
             for nodeid, date, user, action, params in self.history(nodeid):
                 date = date.get_tuple()
                 if action == 'set':
+                    export_data = {}
                     for propname, value in params.items():
+                        if not properties.has_key(propname):
+                            # property no longer in the schema
+                            continue
+
                         prop = properties[propname]
                         # make sure the params are eval()'able
                         if value is None:
@@ -1717,7 +1718,8 @@ class Class(hyperdb.Class):
                             value = value.get_tuple()
                         elif isinstance(prop, Password):
                             value = str(value)
-                        params[propname] = value
+                        export_data[propname] = value
+                    params = export_data
                 l = [nodeid, date, user, action, params]
                 r.append(map(repr, l))
         return r
