@@ -1659,6 +1659,7 @@ class Class(hyperdb.Class):
                 if value is not None and type(value) != type('') and type(value) != type(u''):
                     raise TypeError, 'new property "%s" not a string'%propname
                 if prop.indexme:
+                    if value is None: value = ''
                     self.db.indexer.add_text((self.classname, nodeid, propname),
                         value)
 
@@ -1698,9 +1699,14 @@ class Class(hyperdb.Class):
         propvalues['activity'] = date.Date()
         propvalues['actor'] = self.db.getuid()
 
-        # do the set, and journal it
+        # do the set
         self.db.setnode(self.classname, nodeid, propvalues, multilink_changes)
 
+        # remove the activity props now they're handled
+        del propvalues['activity']
+        del propvalues['actor']
+
+        # journal the set
         if self.do_journal:
             self.db.addjournal(self.classname, nodeid, ''"set", journalvalues)
 
@@ -2046,8 +2052,6 @@ class Class(hyperdb.Class):
 
         cn = self.classname
 
-        timezone = self.db.getUserTimezone()
-
         # vars to hold the components of the SQL statement
         frum = []       # FROM clauses
         loj = []        # LEFT OUTER JOIN clauses
@@ -2136,7 +2140,7 @@ class Class(hyperdb.Class):
                 else:
                     try:
                         # Try to filter on range of dates
-                        date_rng = Range(v, date.Date, offset=timezone)
+                        date_rng = propclass.range_from_raw (v, self.db)
                         if date_rng.from_value:
                             where.append('_%s._%s >= %s'%(cn, k, a))
                             args.append(dc(date_rng.from_value))
