@@ -543,7 +543,6 @@ class EditItemAction(EditCommon):
             message = self._editnodes(props, links)
         except (ValueError, KeyError, IndexError,
                 roundup.exceptions.Reject), message:
-            import traceback;traceback.print_exc()
             self.client.error_message.append(
                 self._('Edit Error: %s') % str(message))
             return
@@ -845,12 +844,18 @@ class LogoutAction(Action):
         # construct the logout cookie
         now = Cookie._getdate()
         self.client.additional_headers['Set-Cookie'] = \
-           '%s=deleted; Max-Age=0; expires=%s; Path=%s;'%(self.client.cookie_name,
-            now, self.client.cookie_path)
+           '%s=deleted; Max-Age=0; expires=%s; Path=%s;' % (
+               self.client.cookie_name, now, self.client.cookie_path)
 
         # Let the user know what's going on
         self.client.ok_message.append(self._('You are logged out'))
 
+        # reset client context to render tracker home page
+        # instead of last viewed page (may be inaccessibe for anonymous)
+        self.client.classname = None
+        self.client.nodeid = None
+        self.client.template = None
+
 class LoginAction(Action):
     def handle(self):
         """Attempt to log a user in.
