@@ -85,10 +85,10 @@ def find_template(dir, name, view):
 
     # try with a .html or .xml extension (new-style)
     for extension in '.html', '.xml':
-        filename = filename + extension
-        src = os.path.join(dir, filename)
+        f = filename + extension
+        src = os.path.join(dir, f)
         if os.path.exists(src):
-            return (src, filename)
+            return (src, f)
 
     # no view == no generic template is possible
     if not view:
@@ -620,7 +620,8 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
         return l
 
     def classhelp(self, properties=None, label=''"(list)", width='500',
-            height='400', property='', form='itemSynopsis'):
+            height='400', property='', form='itemSynopsis',
+            pagesize=50, sort=None, filter=None):
         '''Pop up a javascript window with class help
 
         This generates a link to a popup window which displays the
@@ -630,8 +631,16 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
         properties of a class (excluding id, creator, created and
         activity).
 
-        You may optionally override the label displayed, the width and
-        height. The popup window will be resizable and scrollable.
+        You may optionally override the label displayed, the width,
+        the height, the number of items per page and the field on which
+        the list is sorted (defaults to username if in the displayed
+        properties).
+
+        With the "filter" arg it is possible to specify a filter for
+        which items are supposed to be displayed. It has to be of
+        the format "<field>=<values>;<field>=<values>;...".
+
+        The popup window will be resizable and scrollable.
 
         If the "property" arg is given, it's passed through to the
         javascript help_window function.
@@ -644,13 +653,31 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
             properties = self._klass.getprops(protected=0).keys()
             properties.sort()
             properties = ','.join(properties)
+        if sort is None:
+            if 'username' in properties.split( ',' ):
+                sort = '&amp;@sort=username'
+            else:
+                sort = ''
+        else:
+            sort = '&amp;@sort=' + sort
         if property:
             property = '&amp;property=%s'%property
         if form:
             form = '&amp;form=%s'%form
+        if filter:
+            filterprops = filter.split(';')
+            filtervalues = []
+            names = []
+            for x in filterprops:
+                (name, values) = x.split('=')
+                names.append(name)
+                filtervalues.append('&amp;%s=%s' % (name, urllib.quote(values)))
+            filter = '&amp;@filter=%s%s' % (','.join(names), ''.join(filtervalues))
+        else:
+           filter = ''
         help_url = "%s?@startwith=0&amp;@template=help&amp;"\
-                   "properties=%s%s%s" % \
-                   (self.classname, properties, property, form)
+                   "properties=%s%s%s%s&amp;@pagesize=%s%s" % \
+                   (self.classname, properties, property, form, sort, pagesize, filter)
         onclick = "javascript:help_window('%s', '%s', '%s');return false;" % \
                   (help_url, width, height)
         return '<a class="classhelp" href="%s" onclick="%s">%s</a>' % \
@@ -1055,7 +1082,7 @@ class _HTMLUser(_HTMLItem):
         if classname is self._marker:
             classname = self._client.classname
         return self._client.db.security.hasPermission(permission,
-            self._nodeid, classname)
+            self._nodeid, classname, property, itemid)
 
 def HTMLItem(client, classname, nodeid, anonymous=0):
     if classname == 'user':
@@ -1146,7 +1173,7 @@ class StringHTMLProperty(HTMLProperty):
                 # make sure s1 is a valid tracker classname
                 cl = self._db.getclass(s1)
                 if not cl.hasnode(s2):
-                    raise KeyError, 'oops'
+                    return s
                 return '<a href="%s%s">%s</a>'%(s1, s2, s)
             except KeyError:
                 return s
@@ -1327,7 +1354,7 @@ class BooleanHTMLProperty(HTMLProperty):
 
         if self._value is None:
             return ''
-        return self._value and "Yes" or "No"
+        return self._value and self._("Yes") or self._("No")
 
     def field(self):
         ''' Render a form edit field for the property
@@ -1346,15 +1373,15 @@ class BooleanHTMLProperty(HTMLProperty):
         if value:
             s = self.input(type="radio", name=self._formname, value="yes",
                 checked="checked")
-            s += 'Yes'
+            s += self._('Yes')
             s +=self.input(type="radio", name=self._formname, value="no")
-            s += 'No'
+            s += self._('No')
         else:
             s = self.input(type="radio", name=self._formname, value="yes")
-            s += 'Yes'
+            s += self._('Yes')
             s +=self.input(type="radio", name=self._formname, value="no",
                 checked="checked")
-            s += 'No'
+            s += self._('No')
         return s
 
 class DateHTMLProperty(HTMLProperty):
@@ -1369,6 +1396,8 @@ class DateHTMLProperty(HTMLProperty):
                 isinstance(self._value, unicode)):
             self._value.setTranslator(self._client.translator)
         self._offset = offset
+        if self._offset is None :
+            self._offset = self._prop.offset (self._db)
 
     def plain(self):
         ''' Render a "plain" representation of the property
@@ -1453,8 +1482,11 @@ class DateHTMLProperty(HTMLProperty):
             else:
                 value = date.Date(raw_value).pretty(format)
         else:
-            tz = self._db.getUserTimezone()
-            value = raw_value.local(tz)
+            if self._offset is None :
+                offset = self._db.getUserTimezone()
+            else :
+                offset = self._offset
+            value = raw_value.local(offset)
             if format is not self._marker:
                 value = value.pretty(format)
 
@@ -1510,7 +1542,7 @@ class IntervalHTMLProperty(HTMLProperty):
             anonymous=0):
         HTMLProperty.__init__(self, client, classname, nodeid, prop,
             name, value, anonymous)
-        if self._value:
+        if self._value and not isinstance(self._value, (str, unicode)):
             self._value.setTranslator(self._client.translator)
 
     def plain(self):
@@ -1566,8 +1598,12 @@ class LinkHTMLProperty(HTMLProperty):
 
     def __getattr__(self, attr):
         ''' return a new HTMLItem '''
-       #print 'Link.getattr', (self, attr, self._value)
         if not self._value:
+            # handle a special page templates lookup
+            if attr == '__render_with_namespace__':
+                def nothing(*args, **kw):
+                    return ''
+                return nothing
             msg = self._('Attempt to look up %(attr)s on a missing value')
             return MissingValue(msg%locals())
         i = HTMLItem(self._client, self._prop.classname, self._value)
@@ -1621,8 +1657,10 @@ class LinkHTMLProperty(HTMLProperty):
             "additional" lists properties which should be included in the
                 label
             "sort_on" indicates the property to sort the list on as
-                (direction, property) where direction is '+' or '-'.
-            
+                (direction, property) where direction is '+' or '-'. A
+                single string with the direction prepended may be used.
+                For example: ('-', 'order'), '+name'.
+
             The remaining keyword arguments are used as conditions for
             filtering the items in the list - they're passed as the
             "filterspec" argument to a Class.filter() call.
@@ -1641,13 +1679,16 @@ class LinkHTMLProperty(HTMLProperty):
         if value is None:
             s = 'selected="selected" '
         l.append(self._('<option %svalue="-1">- no selection -</option>')%s)
-        if linkcl.getprops().has_key('order'):
-            sort_on = ('+', 'order')
+
+        if sort_on is not None:
+            if not isinstance(sort_on, tuple):
+                if sort_on[0] in '+-':
+                    sort_on = (sort_on[0], sort_on[1:])
+                else:
+                    sort_on = ('+', sort_on)
         else:
-            if sort_on is None:
-                sort_on = ('+', linkcl.labelprop())
-            else:
-                sort_on = ('+', sort_on)
+            sort_on = ('+', find_sort_key(linkcl))
+
         options = linkcl.filter(None, conditions, sort_on, (None, None))
 
         # make sure we list the current value if it's retired
@@ -1698,10 +1739,17 @@ class MultilinkHTMLProperty(HTMLProperty):
     def __init__(self, *args, **kwargs):
         HTMLProperty.__init__(self, *args, **kwargs)
         if self._value:
-            self._value = lookupIds(self._db, self._prop, self._value,
+            display_value = lookupIds(self._db, self._prop, self._value,
                 fail_ok=1)
             sortfun = make_sort_function(self._db, self._prop.classname)
-            self._value.sort(sortfun)
+            # sorting fails if the value contains
+            # items not yet stored in the database
+            # ignore these errors to preserve user input
+            try:
+                display_value.sort(sortfun)
+            except:
+                pass
+            self._value = display_value
 
     def __len__(self):
         ''' length of the multilink '''
@@ -1793,8 +1841,10 @@ class MultilinkHTMLProperty(HTMLProperty):
             "additional" lists properties which should be included in the
                 label
             "sort_on" indicates the property to sort the list on as
-                (direction, property) where direction is '+' or '-'.
-            
+                (direction, property) where direction is '+' or '-'. A
+                single string with the direction prepended may be used.
+                For example: ('-', 'order'), '+name'.
+
             The remaining keyword arguments are used as conditions for
             filtering the items in the list - they're passed as the
             "filterspec" argument to a Class.filter() call.
@@ -1807,10 +1857,16 @@ class MultilinkHTMLProperty(HTMLProperty):
         value = self._value
 
         linkcl = self._db.getclass(self._prop.classname)
-        if sort_on is None:
-            sort_on = ('+', find_sort_key(linkcl))
+
+        if sort_on is not None:
+            if not isinstance(sort_on, tuple):
+                if sort_on[0] in '+-':
+                    sort_on = (sort_on[0], sort_on[1:])
+                else:
+                    sort_on = ('+', sort_on)
         else:
-            sort_on = ('+', sort_on)
+            sort_on = ('+', find_sort_key(linkcl))
+
         options = linkcl.filter(None, conditions, sort_on)
         height = height or min(len(options), 7)
         l = ['<select multiple name="%s" size="%s">'%(self._formname, height)]
@@ -2199,7 +2255,10 @@ function help_window(helpurl, width, height) {
         klass = self.client.db.getclass(self.classname)
         if self.search_text:
             matches = self.client.db.indexer.search(
-                re.findall(r'\b\w{2,25}\b', self.search_text), klass)
+                [w.upper().encode("utf-8", "replace") for w in re.findall(
+                    r'(?u)\b\w{2,25}\b',
+                    unicode(self.search_text, "utf-8", "replace")
+                )], klass)
         else:
             matches = None
 
@@ -2275,8 +2334,11 @@ class Batch(ZTUtils.Batch):
         ''' Detect if the property marked as being the group property
             changed in the last iteration fetch
         '''
+        # we poke directly at the _value here since MissingValue can screw
+        # us up and cause Nones to compare strangely
         if (self.last_item is None or
-                self.last_item[property] != self.current_item[property]):
+                self.last_item[property]._value !=
+                self.current_item[property]._value):
             return 1
         return 0
 
