@@ -467,6 +467,14 @@ SETTINGS = (
             "by OS environment variable LANGUAGE, LC_ALL, LC_MESSAGES,\n"
             "or LANG, in that order of preference."),
     )),
+    ("web", (
+        (Option, 'http_auth', "yes",
+            "Whether to use HTTP Basic Authentication, if present.\n"
+            "Roundup will use either the REMOTE_USER or HTTP_AUTHORIZATION\n"
+            "variables supplied by your web server (in that order).\n"
+            "Set this option to 'no' if you do not wish to use HTTP Basic\n"
+            "Authentication in your web interface."),
+    )),
     ("rdbms", (
         (Option, 'name', 'roundup',
             "Name of the database to use.",
