@@ -81,7 +81,7 @@ import time, random, sys, logging
 import traceback, MimeWriter, rfc822
 
 from roundup import hyperdb, date, password, rfc2822, exceptions
-from roundup.mailer import Mailer
+from roundup.mailer import Mailer, MessageSendError
 
 SENDMAILDEBUG = os.environ.get('SENDMAILDEBUG', '')
 
@@ -546,7 +546,19 @@ class MailGW:
             # just inform the user that he is not authorized
             m = ['']
             m.append(str(value))
-            self.mailer.bounce_message(message, [sendto[0][1]], m)
+            try:
+                self.mailer.bounce_message(message, [sendto[0][1]], m)
+            except MessageSendError, error:
+                # if the only reason the bounce failed is because of
+                # invalid addresses to bounce the message back to, then
+                # just discard the message - it's just not worth bothering
+                # with (most likely spam / otherwise forged)
+                invalid = True
+                for address, (code, reason) in error.keys():
+                    if code != 550:
+                        invalid = False
+                if not invalid:
+                    raise
         except IgnoreMessage:
             # do not take any action
             # this exception is thrown when email should be ignored
@@ -579,11 +591,21 @@ class MailGW:
         if message.getheader('x-roundup-loop', ''):
             raise IgnoreLoop
 
+        # handle the subject line
+        subject = message.getheader('subject', '')
+        if not subject:
+            raise MailUsageError, '''
+Emails to Roundup trackers must include a Subject: line!
+'''
+
         # detect Precedence: Bulk, or Microsoft Outlook autoreplies
         if (message.getheader('precedence', '') == 'bulk'
-            or message.getheader('subject', '').lower().find("autoreply") > 0):
+                or subject.lower().find("autoreply") > 0):
             raise IgnoreBulk
 
+        if subject.strip().lower() == 'help':
+            raise MailUsageHelp
+
         # config is used many times in this method.
         # make local variable for easier access
         config = self.instance.config
@@ -609,20 +631,8 @@ class MailGW:
         if not from_list:
             from_list = message.getaddrlist('from')
 
-        # handle the subject line
-        subject = message.getheader('subject', '')
-
-        if not subject:
-            raise MailUsageError, '''
-Emails to Roundup trackers must include a Subject: line!
-'''
-
-        if subject.strip().lower() == 'help':
-            raise MailUsageHelp
-
-        m = self.subject_re.match(subject)
-
         # check for well-formed subject line
+        m = self.subject_re.match(subject)
         if m:
             # get the classname
             classname = m.group('classname')
