@@ -38,8 +38,8 @@ config.TRACKER_WEB = "http://tracker.example/cgi-bin/roundup.cgi/bugs/"
 # uncomment the following to have excessive debug output from test cases
 # FIXME: tracker logging level should be increased by -v arguments
 #   to 'run_tests.py' script
-config.LOGGING_FILENAME = "/tmp/logfile"
-config.LOGGING_LEVEL = "DEBUG"
+#config.LOGGING_FILENAME = "/tmp/logfile"
+#config.LOGGING_LEVEL = "DEBUG"
 config.init_logging()
 
 def setupTracker(dirname, backend="anydbm"):
