@@ -21,7 +21,7 @@ if not os.environ.has_key('SENDMAILDEBUG'):
 SENDMAILDEBUG = os.environ['SENDMAILDEBUG']
 
 from roundup.mailgw import MailGW, Unauthorized, uidFromAddress, \
-    parseContent, IgnoreLoop, IgnoreBulk
+    parseContent, IgnoreLoop, IgnoreBulk, MailUsageError
 from roundup import init, instance, password, rfc2822, __version__
 
 import db_test_base
@@ -1018,6 +1018,18 @@ Message-Id: <dummy_test_message_id>
 Subject: Re: [issue] Out of office AutoReply: Back next week
 
 Hi, I'm back in the office next week
+''')
+
+    def testNoSubject(self):
+        self.assertRaises(MailUsageError, self._handle_mail,
+            '''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: Chef <chef@bork.bork.bork>
+To: issue_tracker@your.tracker.email.domain.example
+Cc: richard@test
+Reply-To: chef@bork.bork.bork
+Message-Id: <dummy_test_message_id>
+
 ''')
 
 def test_suite():
